package net.knarcraft.ffmpegconverter.container;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * A record for storing the result of probing for streams
 *
 * @param parsedFiles   <p>The files that were parsed to get the attached streams</p>
 * @param parsedStreams <p>The streams that were parsed from the files</p>
 */
public record StreamProbeResult(@NotNull List<File> parsedFiles, @NotNull List<StreamObject> parsedStreams) {

    /**
     * Gets all probed subtitle streams
     *
     * @return <p>All probed subtitle streams</p>
     */
    @NotNull
    public List<SubtitleStream> getSubtitleStreams() {
        return filterStreamsByType(this.parsedStreams, SubtitleStream.class);
    }

    /**
     * Gets all probed audio streams
     *
     * @return <p>All probed audio streams</p>
     */
    @NotNull
    public List<AudioStream> getAudioStreams() {
        return filterStreamsByType(this.parsedStreams, AudioStream.class);
    }

    /**
     * Gets all probed video streams
     *
     * @return <p>All probed video streams</p>
     */
    @NotNull
    public List<VideoStream> getVideoStreams() {
        return filterStreamsByType(this.parsedStreams, VideoStream.class);
    }

    /**
     * Filters parsed streams into one of the stream types
     *
     * @param streams <p>A list of stream objects.</p>
     * @param clazz   <p>The class to filter</p>
     * @param <G>     <p>The correct object type for the streams with the selected codec type.</p>
     * @return <p>A potentially shorter list of streams.</p>
     */
    @NotNull
    @SuppressWarnings("unchecked")
    private <G extends StreamObject> List<G> filterStreamsByType(@NotNull List<StreamObject> streams,
                                                                 @NotNull Class<?> clazz) {
        List<G> newStreams = new ArrayList<>();
        for (StreamObject stream : streams) {
            if (stream.getClass() == clazz) {
                newStreams.add((G) stream);
            }
        }
        return newStreams;
    }

}
