package net.knarcraft.ffmpegconverter.converter.module.mapping;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A module for selecting one or more audio streams, sorted by language
 */
public class SetDefaultStreamModule<K extends StreamObject> implements ConverterModule {

    private final List<K> streams;
    private final int defaultStream;

    /**
     * Instantiates a new language sorted audio stream module
     *
     * @param streams       <p>All input streams</p>
     * @param defaultStream <p>The index of the output stream to set as default</p>
     */
    public SetDefaultStreamModule(@NotNull List<K> streams, int defaultStream) {
        this.streams = streams;
        this.defaultStream = defaultStream;
    }

    @Override
    public void addArguments(@NotNull FFMpegCommand command) {
        for (int i = 0; i < streams.size(); i++) {
            K stream = streams.get(i);
            char defaultModifier;
            if (i == defaultStream) {
                defaultModifier = '+';
            } else {
                defaultModifier = '-';
            }
            command.addOutputFileOption(String.format("-disposition:%s:%d", stream.streamTypeCharacter(), i),
                    String.format("%sdefault", defaultModifier));
        }
    }

}
