package net.knarcraft.ffmpegconverter.streams;

import net.knarcraft.ffmpegconverter.utility.ValueParsingHelper;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * An abstract implementation of a stream object implementing common methods
 */
public abstract class AbstractStream implements StreamObject {

    protected final int inputIndex;
    protected final int absoluteIndex;
    protected final int relativeIndex;
    protected final String codecName;
    protected final String language;
    protected final boolean isDefault;
    protected final String title;

    /**
     * Instantiates a new abstract stream
     *
     * @param streamInfo    <p>All info about the stream</p>
     * @param inputIndex    <p>The index of the input file this stream belongs to</p>
     * @param relativeIndex <p>The relative index of this stream, only considering streams of the same type</p>
     */
    protected AbstractStream(@NotNull Map<StreamTag, String> streamInfo, int inputIndex, int relativeIndex) {
        this.codecName = ValueParsingHelper.parseString(streamInfo.get(StreamTag.CODEC_NAME), "");
        this.absoluteIndex = ValueParsingHelper.parseInt(streamInfo.get(StreamTag.INDEX), -1);
        this.language = ValueParsingHelper.parseString(streamInfo.get(StreamTag.TAG_LANGUAGE), "und");
        this.isDefault = ValueParsingHelper.parseBoolean(streamInfo.get(StreamTag.DISPOSITION_DEFAULT), false);
        this.title = ValueParsingHelper.parseString(streamInfo.get(StreamTag.TAG_TITLE), "");
        this.inputIndex = inputIndex;
        this.relativeIndex = relativeIndex;
    }

    @Override
    public @NotNull String getCodecName() {
        return this.codecName;
    }

    @Override
    public int getAbsoluteIndex() {
        return this.absoluteIndex;
    }

    @Override
    public int getRelativeIndex() {
        return this.relativeIndex;
    }

    @Override
    public @NotNull String getLanguage() {
        return this.language;
    }

    @Override
    public boolean isDefault() {
        return this.isDefault;
    }

    @Override
    @NotNull
    public String getTitle() {
        return this.title;
    }

    @Override
    public int getInputIndex() {
        return this.inputIndex;
    }

}