package net.knarcraft.ffmpegconverter.streams;

import net.knarcraft.ffmpegconverter.utility.ValueParsingHelper;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * This class represents a ffmpeg audio stream
 */
public class AudioStream extends AbstractStream implements StreamObject {

    private final int channels;

    /**
     * Instantiates a new audio stream
     *
     * @param streamInfo    <p>All info about the stream</p>
     * @param inputIndex    <p>The index of the input file containing this stream</p>
     * @param relativeIndex <p>The index of the audio stream relative to other audio streams.</p>
     */
    public AudioStream(@NotNull Map<StreamTag, String> streamInfo, int inputIndex, int relativeIndex) {
        super(streamInfo, inputIndex, relativeIndex);
        this.channels = ValueParsingHelper.parseInt(streamInfo.get(StreamTag.CHANNELS), 0);
    }

    /**
     * Gets the number of channels for the audio stream
     *
     * @return <p>The number of channels for the audio stream.</p>
     */
    public int getChannels() {
        return this.channels;
    }

    @Override
    public char streamTypeCharacter() {
        return 'a';
    }

}
